﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MPEIVILib;
using Ivi.DCPwr.Interop;
using Ivi.Driver.Interop;

namespace MPE_IVI_Driver_Example
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                IMPEIVI driver = null;
                try
                {
                    driver = new MPEIVI();
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Error creating driver object.  Ensure IVI driver is installed.");
                    Console.WriteLine("Exception Details: " + ex.ToString());
                    return;
                }

                try
                {

                    //output basic driver information
                    Console.WriteLine("Identifier: {0}", driver.Identity.Identifier);
                    Console.WriteLine("Revision: {0}", driver.Identity.Revision);
                    Console.WriteLine("Vendor: {0}", driver.Identity.Vendor);
                    Console.WriteLine("Description: {0}", driver.Identity.Description);

                    // Setup VISA resource descriptor - how the driver will connect to the power supply.

                    //Examples of other valid VISA resource descriptors
                    //"TCPIP::MPE12345678::50505::SOCKET";   //NETBIOS name direct to TCP port
                    //"TCPIP::MPE12345678::inst0::INSTR";    //NETBIOS name to VXI (via UDP and TCP port 111)
                    //"TCPIP::192.168.1.100::50505::SOCKET"; //IP address direct to TCP port
                    //"GPIB0::13::INSTR";                    //GPIB address 13
                    //"ASRL1";                               //COM Port 1


                    string resourceDesc = "TCPIP::MPE00000361::50505::SOCKET";


                    // Setup IVI-defined initialization options
                    string standardInitOptions =
                        "Cache=true, InterchangeCheck=false, QueryInstrStatus=true, RangeCheck=true, RecordCoercions=false, Simulate=false";

                    // Setup driver-specific initialization options
                    string driverSetupOptions =
                        "DriverSetup= Model=MPEIVI, Trace=false";

                    Console.WriteLine("Connecting to Power Supply...");
                    driver.Initialize(resourceDesc, true, false, standardInitOptions + "," + driverSetupOptions);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Error initializing driver.");
                    Console.WriteLine("Ensure resource descriptor is correct and power supply is on.");
                    Console.WriteLine("Exception Message:" + ex.Message);
                    return;
                }


                try
                {
                    //Retrieve the name of the only output of the power supply
                    string outputName = driver.Outputs.get_Name(1);

                    //There are two interfaces for accessing the power supply
                    //IIviDCPwrOutput and IMPEIVIOutput.
                    //
                    //The IVI DCPwr interface can be used to access all of the standard IVI-defined functions
                    //The MPE interface allows access to the full functionality of the power supply.

                    IMPEIVIOutput mpeOutput = driver.Outputs.get_Item(outputName) as IMPEIVIOutput;
                    IIviDCPwrOutput iviOutput = mpeOutput;

                    Console.WriteLine("Making sure Power supply is in remote mode.");
                    MPEIVISetpointEnum origSetPoint = mpeOutput.Setpoint;

                    //Make sure the power supply is off
                    mpeOutput.Enabled = false;

                    //Ensure the setpoint is Remote
                    mpeOutput.Setpoint = MPEIVISetpointEnum.MPEIVISetpointRemote;


                    //issue a *RST
                    (driver as IIviDriverUtility).Reset();

                    //Retrieve the IDN string
                    string idn = mpeOutput.GetIDN();
                    Console.WriteLine(idn);

                    //Retrieve the current and voltage scale
                    double currScale = mpeOutput.CalGetCurrentScale();  //equivalent to QueryCurrentLimitMax()
                    double voltScale = mpeOutput.CalGetVoltageScale();  //equivalent to QueryVoltageLimitMax()

                    Console.WriteLine("Max voltage: " + voltScale);
                    Console.WriteLine("Max current: " + currScale);
                    


                }
                catch (Exception ex)
                {

                    Console.WriteLine("\t" + ex.Message);
                    int errorNum = -1;
                    string errorMsg = null;
                    Console.WriteLine("");
                    while (errorNum != 0)
                    {
                        driver.Utility.ErrorQuery(ref errorNum, ref errorMsg);
                        Console.WriteLine("\tErrorQuery: {0}, {1}", errorNum, errorMsg);
                    }

                }
            }
            finally
            {
                Console.WriteLine("Press Enter key to exit.");
                Console.ReadLine();
            }
        }
    }
}